﻿Type=Class
Version=4.01
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
#Event:MouseClicked (EventData As MouseEvent)

'Class module
Sub Class_Globals
'	Private fx As JFX
	Private wv As WebView
	
	Private sImagePath As String
	Private sObjectFit = "CONTAIN" As String
	Private sObjectPosition = $"50% 50%"$ As String
	Private sImageOrientation = $"0deg"$ As String
	Private oModule As Object
	Private sEventName As String	
	
	Private sColorBack As String = "black"
	Private sColorLink As String = "white"
End Sub

'Initialize the WebImageView.
'
'Parent: An anchorpane where the image will be put
'Module: the module that will receive the events (usually "Me")
'EventName: The name of the event
Public Sub Initialize(Parent As AnchorPane, Module As Object, EventName As String)
	oModule = Module
	sEventName = EventName
	
	wv.Initialize("wv")	
	Parent.RemoveAllNodes
	Parent.AddNode(wv, 0, 0, -1, -1)
	Parent.SetAnchors(wv, 0, 0, 0, 0)
	
	'This is so that the mouse scroll is not caught by the webview
	'create the click event on the anchorpane:
	asJo(wv).RunMethod("setMouseTransparent", Array(True))
	Dim e As Object = asJo(Parent).CreateEvent("javafx.event.EventHandler", "apParent_OnMouseClicked", False)
	asJo(Parent).RunMethod("setOnMouseClicked", Array(e))
		
	LoadImageDelayed
End Sub

'Set a tag associated with this class.
Public Sub setTag(t As Object)
	wv.Tag = t
End Sub
Public Sub getTag As Object
	Return wv.Tag
End Sub

'Pass a file name or url.
'Do not forget to call LoadImage or LoadImageDelayed to display the image.
Public Sub SetImage(Path As String)
	sImagePath = Path
End Sub

'Sets the background color (behind the image).
Public Sub setBackColor(Color As String)
	sColorBack = Color
End Sub
Public Sub getBackColor As String
	Return sColorBack
End Sub

'How the image fits in the pane (default:CONTAIN):
'CONTAIN: will fit inside the dimensions of the pane
'FILL: Causes the image to expand and completely fill the pane, even if that break the aspect ratio.
'COVER: Respects the image ratio but will fill all empty space.
'NONE: Completely ignores the height and width of the pane.
Public Sub setImageFit(Fit As String)
	Dim Accepted As List = Array As String("CONTAIN", "FILL", "COVER", "NONE")		
	If Accepted.IndexOf(Fit.ToUpperCase) > -1 Then
		sObjectFit = Fit.ToUpperCase
	Else
		Log($"Invalid Fit Value: ${Fit}"$)
		sObjectFit = "CONTAIN"	
	End If
End Sub
Public Sub getImageFit As String
	Return sObjectFit
End Sub

'Sets the position of the image in the pane (default:50% 50%):
'2 values separated by a space, example:
'50% 50%
'100px 50px
'
'Or one of the following:
'INHERIT
'INITIAL
'UNSET
'
'More values here:
'<link>"object-position"|https://developer.mozilla.org/en-US/docs/Web/CSS/object-position</link>
Public Sub setImagePosition(Position As String)
	sObjectPosition = Position
End Sub
Public Sub getImagePosition As String
	Return sObjectPosition
End Sub

'Sets the orientation of the image in the pane (default:0deg):
'example:
'0deg
'6.4deg
'from-image
'90deg flip
'flip
'
'Or one of the following:
'INHERIT
'INITIAL
'UNSET
'
'More values here:
'<link>"image-orientation"|https://developer.mozilla.org/en-US/docs/Web/CSS/image-orientation</link>
Public Sub setImageOrientation(Orientation As String)
	sImageOrientation = Orientation
End Sub
Public Sub getImageOrientation As String
	Return sImageOrientation
End Sub

'Loads the image without blocking the code execution.
Public Sub LoadImageDelayed
	CallSubDelayed(Me, "LoadImage")
End Sub

'Loads the image
Public Sub LoadImage
	Dim Html As String
	Html = $"
	<!DOCTYPE html>
	<html>
		<head>
			<style Type='text/css'>
			html, body
			{
				background-color: ${sColorBack};
				margin: 0;
				width: 100%;
				height: 100%;
				overflow-x: hidden;
				overflow-y: hidden;
			}
			.image
			{
				background-color: ${sColorBack};
				padding: 0px;
				overflow-x: hidden;
				overflow-y: hidden;
				width: 100%;
				height: 100%;				
				margin: 0px;
			 	color: ${sColorBack};
				Font-size: 0.8em;
				Font-family: arial;
			}
			a:link {color: ${sColorLink};} 
			a:visited {color: derive(${sColorLink}, -20%);}
			a:hover {color: derive(${sColorLink}, 20%);}
			a:active {color: derive(${sColorLink}, 40%);}
			</style>
		</head>
		<body>
			<div class='image' id='image'>
				<img src="${iif(sImagePath.ToLowerCase.StartsWith("http"), sImagePath, File.GetUri(File.GetFileParent(sImagePath), File.GetName(sImagePath)))}"
					alt="HTML5 Icon"
					style= "width: 100%; height: 100%; border: 0;
						object-position: ${sObjectPosition};
						object-fit: ${sObjectFit};
						image-orientation: ${sImageOrientation};
					"
				>
			</div>
		</body>
	</Html>
	"$
	
	wv.LoadHtml(Html)
End Sub

'Private Sub wv_PageFinished (Url As String)
'	'...
'End Sub

Private Sub wv_MouseClicked (EventData As MouseEvent)
	If oModule <> Null And sEventName <> "" Then
		If SubExists(oModule, $"${sEventName}_MouseClicked"$) Then
			CallSubDelayed2(oModule, $"${sEventName}_MouseClicked"$, EventData)
		End If
	End If
End Sub

Sub apParent_OnMouseClicked_Event(MethodName As String, Args() As Object)
	If Args.Length > 0 Then
		Dim e As MouseEvent = Args(0)
		wv_MouseClicked(e)
	End If	
End Sub

Private Sub asJo(o As JavaObject) As JavaObject
	Return o
End Sub

Private Sub iif(Condition As Boolean, vTrue As Object, vFalse As Object) As Object
	If Condition Then Return vTrue
	Return vFalse
End Sub